/*BEGIN_LEGAL 
Intel Open Source License 

Copyright (c) 2002-2015 Intel Corporation. All rights reserved.
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.  Redistributions
in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.  Neither the name of
the Intel Corporation nor the names of its contributors may be used to
endorse or promote products derived from this software without
specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INTEL OR
ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
END_LEGAL */
/*
 * This test verifies that you can flush the code cache while calling
 * an application function.  Previously, this test caused a crash because
 * the flush would free the trace that contained the call to MallocWrapper().
 * Thus, a crash happened when PIN_CallApplicationFunction() returned back.
 */

#include <iostream>
#include "pin.H"

static VOID ImageLoad(IMG, VOID *);
static VOID *MallocWrapper(const CONTEXT *, THREADID, AFUNPTR, size_t);


int main(int argc, char **argv)
{
    PIN_Init(argc, argv);

#if defined(TARGET_MAC)
    PIN_InitSymbols();
#endif

    IMG_AddInstrumentFunction(ImageLoad, 0);
    PIN_StartProgram();

    return 0;
}

static VOID ImageLoad(IMG img, VOID *)
{
    PROTO proto_malloc = PROTO_Allocate(PIN_PARG(void *), CALLINGSTD_DEFAULT,
        "malloc", PIN_PARG(size_t), PIN_PARG_END());

    RTN rtn = RTN_FindByName(img, "malloc");
    if (RTN_Valid(rtn))
    {
        std::cout << "Replacing malloc in " << IMG_Name(img) << endl;

        RTN_ReplaceSignature(rtn, AFUNPTR(MallocWrapper),
            IARG_PROTOTYPE, proto_malloc,
            IARG_CONST_CONTEXT,
            IARG_THREAD_ID,
            IARG_ORIG_FUNCPTR,
            IARG_FUNCARG_ENTRYPOINT_VALUE, 0,
            IARG_END);
    }
}


static VOID *MallocWrapper(const CONTEXT *context, THREADID tid, AFUNPTR origMalloc, size_t size)
{
    CODECACHE_FlushCache();

    VOID *ret;
    PIN_CallApplicationFunction(context, tid, CALLINGSTD_DEFAULT, origMalloc, NULL,
        PIN_PARG(void *), &ret,
        PIN_PARG(size_t), size,
        PIN_PARG_END());

    CODECACHE_FlushCache();
    return ret;
}
